<?php
/**
 *    This file is part of OXIDJson.
 *
 *    OXIDJson is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    OXIDJson is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this package.  If not, see <http://www.gnu.org/licenses/>.
 */

use Tonic\Resource;

/**
 * Class OxRestBase
 */
class OxRestBase extends Resource {

    /**
     *
     * @var bool Activate logging
     */
    protected $_debug = true;

    /**
     *
     * @var oxUser Logged in user.
     */
    protected $_oUser = null;


    /**
     * @throws Tonic\UnauthorizedException
     */
    public function setup() {
        $oSession = oxRegistry::getSession();
        $oUtilsServer = oxRegistry::get('oxUtilsServer');
        $oUser = oxNew('oxuser');
        // get auth header and check login data submitted
        $authHeader = $oUtilsServer->getServerVar('HTTP_AUTHORIZATION');
        if (!isset($authHeader) || '' === $authHeader) {
            $authHeader = $oUtilsServer->getServerVar('REDIRECT_HTTP_AUTHORIZATION');
        }

        // Auth comes as "Ox <base64_encoded username:password>"...
        // So strip the "Ox " prefix, base64_decode the remaining string
        // and split username and password at the ":" char

        // also strip auth method "basic" if provided

        if (0 === strpos($authHeader, 'Basic ')) {
            $authHeader = substr($authHeader, 6);
        }
        if (0 === strpos($authHeader, 'Ox')) {
            $authHeader = substr($authHeader, 2);
        }

        $userNamePassword = explode(":", base64_decode($authHeader));

        if (!is_array($userNamePassword) || count($userNamePassword) < 2) {
            throw new Tonic\UnauthorizedException('Malformated password ' . serialize($userNamePassword));
        }
        $sUser = $userNamePassword[0];
        $sPassword = $userNamePassword[1];
        try {
            if (!$oSession->isSessionStarted()) {
                $oSession->start();
            }
            if ($oUser->login($sUser, $sPassword)) {
                // set user
                $this->_oUser = $oUser;

                // check groups
                if (!$this->hasValidLogin()) {
                    $this->_doLog("Login NOT successful for user $sUser");
                    throw new Tonic\UnauthorizedException("Login NOT successful for user $sUser :" . __LINE__);
                } else {

                    $this->_doLog("Login successful for user $sUser - " . $oSession->getVariable('usr'));
                    $oSession->regenerateSessionId();
                }
            }
        } catch (Exception $ex) {
            $this->_doLog("Login NOT successful for user $sUser, Error: " . $ex->getMessage());
            throw new Tonic\UnauthorizedException;
        }
    }

    /**
     * Is the user authorized to use OXJSON?
     * @param oxUser $oUser
     * @return boolean
     */
    protected function hasValidLogin($oUser = null) {
        if(!$this->_oUser && !$oUser) {
            return false;
        }
        if(!$oUser) {
            $oUser = $this->_oUser;
        }
        $isAdmin = $oUser->inGroup('oxidadmin');
        $isRoUser = $oUser->inGroup('oxjsonro');
        $isFullUser = $oUser->inGroup('oxjsonfull');
        if( $isAdmin || $isRoUser || $isFullUser ) {
            return true;
        }
        return false;
    }

    /**
     * Is the user authorized for full access (CRUD)?
     * @return boolean
     */
    protected function hasFullAccess() {
        if(!$this->_oUser) {
            return false;
        }
        $isAdmin = $this->_oUser->inGroup('oxidadmin');
        $isFullUser = $this->_oUser->inGroup('oxjsonfull');
        if( $isAdmin || $isFullUser ) {
            return true;
        }
        return false;
    }

    /**
     * Check access rights for POST, PUT, DELETE
     * @throws Tonic\ConditionException
     */
    public function hasRwAccessRights() {
        if(!$this->hasFullAccess()) {
            throw new Tonic\ConditionException;
        }
    }

    /**
     * Condition method to turn output into JSON.
     *
     * This condition sets a before and an after filter for the request and response. The
     * before filter decodes the request body if the request content type is JSON, while the
     * after filter encodes the response body into JSON.
     */
    protected function json() {
        $this->before(function ($request) {
                    if ($request->contentType == "application/json") {
                        $request->data = json_decode($request->data);
                    }
                });
        $this->after(function ($response) {
                    $response->contentType = "application/json";
                    if (isset($_GET['jsonp'])) {
                        $response->body = $_GET['jsonp'] . '(' . json_encode($response->body) . ');';
                    } else {
                        $response->body = json_encode($response->body);
                    }
                });
    }

    /**
     * Convert an OXID object to an array
     * Hide some specific fields, for now password and passsalt of oxusers
     * @param object $o
     * @return array
     */
    protected function _oxObject2Array($o) {

        $vars = get_object_vars($o);
        $a = array();
        foreach ($vars as $key => $value) {

            $vars = get_object_vars($o);
            foreach ($vars as $key => $value) {
                if (($pos = strpos($key, '__')) > 0) {
                    $key = substr($key, $pos + 2);
                    if (
                            strpos(strtolower($key), 'password') > 0 ||
                            strpos(strtolower($key), 'passsalt') > 0
                    ) {
                        continue;
                    }
                    $value = $value->getRawValue();
                    $a[$key] = $value;
                }
            }
        }
        return $a;
    }

    /**
     * Convert stdClass to array
     * @param object $d
     * @return array
     */
    protected function _objectToArray($d) {
            if (is_object($d)) {
                // Gets the properties of the given object
                // with get_object_vars function
                $d = get_object_vars($d);
            }

            if (is_array($d)) {
                // for recursive call
                return array_map(array('oxRestBase','_objectToArray'), $d);
            }
            else {
                // Return array
                return $d;
            }
    }

    /**
     * General logging function
     * @param string $msg
     * @param string $filename
     */
    protected function _doLog($msg, $filename = "rest.log") {
        if ($this->_debug) {
            oxRegistry::getUtils()->writeToLog(date("Y-m-d H:i:s") . " " . $msg . "\n", $filename);
        }
    }


    protected function erx_get_field_list($class) {

        include 'erx_config_fields.php';

        if (isset($fields[$class])){
            return implode(', ', $fields[$class]);
        } else {
            return ' * ';
        }

    }

    protected function erx_get_class_alias($class) {

        include 'erx_config_fields.php';

        if (isset($aliases[$class])){
            return $aliases[$class];
        } elseif (isset($aliases['ox' . $class]))  {
            return $aliases['ox' . $class];
        } elseif (substr($class,0,2) != 'ox') {
            return 'ox' . $class;
        } else {
            return $class;
        }

    }

    protected function erx_check_is_class_allowed($class) {

        include 'erx_config_fields.php';

        if (!in_array($class, $allowed_classes)) {
            throw new Tonic\ConditionException('Forbidden Class ' . $class . '. Allowed classes: ' . join(', ', $allowed_classes));
        }


    }

    protected function erx_get_select_string($class, $table) {

        $table = mysql_real_escape_string($table);

        switch ($class) {

            case 'oxuser':
                $sql = 'select oxuser.*, IF(oxnewssubscribed.OXDBOPTIN = 1,1,0) AS newsletter from oxuser LEFT JOIN oxnewssubscribed ON (oxuser.oxid = oxnewssubscribed.oxuserid)';
                break;

            case 'oxarticles':
            case 'oxarticle':
            case 'oxarticlelist':
                $sql = 'select oxarticles.*, oxlongdesc, oxtags from oxarticles LEFT JOIN oxartextends ON (oxarticles.oxid = oxartextends.oxid)';
                break;

            case 'oxorder':
                $sql = 'select oxorder.*, bc.oxisoalpha3 as bill_country, dc.oxisoalpha3 as delivery_country
                        from oxorder
                        LEFT JOIN oxcountry AS bc ON ( oxorder.oxbillcountryid = bc.oxid )
                        LEFT JOIN oxcountry AS dc ON ( oxorder.oxdelcountryid = dc.oxid )  ';
                break;

            default:
                $sql = "select " . $this->erx_get_field_list($class) . " from " . $table;
        }

        return $sql;

    }


}
